// Shock Yourself
// by MissEshock2002

// This script, from https://ss.deviatenow.com, is protected by this licence :
// CC by-NC, see http://creativecommons.org/licenses/by-nc/3.0/

setInfos(9, "Shock Yourself", "A punishment program where you have to apply shocks to yourself.",
        "MissEshock2002",
        "v1.8.1", 0x222222, "en", ["estim", "electro", "formale", "forfemale"]);

/*
TODO hideNextShockStats
TODO hidePenaltyStats
TODO hideFinialPunishmentStats
*/

// defaults
String SOUNDSDIR = "ShockYourself/"
String EXT = ".wav"

List<Integer> PAINLEVELRANGELIMIT = [1, 20]
List<Float> SHOCKTIMERANGELIMIT = [0.1, 10.0]

List<Integer> APPLYSHOCKSRANGE = [10, 20]
List<Integer> PAINLEVELRANGE = [5, 15]
List<Float> APPLYTIMERANGE = [10.0, 5.0]
List<Float> SHOCKTIMERANGE = [1.0, 5.0]
List<Float> PAUSETIMERANGE = [2.5, 0.5]
List<Float> DELAYTIMERANGE = [0.0, 0.0]
List<Float> FINALPUNISHMENTMULTIRANGE = [1.0, 1.0]

int APPLYSHOCKPENALTY = 1
int PENALTYSHOCKS = 1
int PENALTYSHOCKSPLUS = 1
int PENALTYPAINLEVELPLUS = 1
float PENALTYSHOCKTIMEPLUS = 0.0
boolean PENALTYLIMIT = true
List FINALPUNISHMENTS = ["missed", "applied", "none"]
boolean AUTOAPPLY = false

// vars
List<Integer> applyShocksRange = []
List<Integer> painLevelRange = []
List<Float> applyTimeRange = []
List<Float> shockTimeRange = []
List<Float> pauseTimeRange = []
List<Float> delayTimeRange = []
List<Float> finalPunishmentMultiRange = []
int applyShocksPenalty
int penaltyShocks // penalty shocks per missed shock
int penaltyShocksPlus
int penaltyPainLevelPlus
float penaltyShockTimePlus
boolean penaltyLimit
String finalPunishment
List estimFileNames = []
boolean autoApply


def getEstimFileNames = {
    List<String> fileNames = []
    List<String> uniqueNames = []
    File dir = new File("sounds/" + SOUNDSDIR)
    dir.traverse(type: groovy.io.FileType.FILES, maxDepth: 0) { file ->
        if (file.name.endsWith(EXT)) fileNames.add(file.name)
        String name = file.name.split('\\.')[0]

        List find = name.findAll('\\d+$')
        if (!find.isEmpty()) {
            if (find[0].isNumber()) {
                String uName = name.replaceAll('\\d+$', '')
                if (!(uName in uniqueNames)) uniqueNames.add(uName)
            }
        }
    }
    List<String> missingNames = []
    uniqueNames.each { uName ->
        for (int lvl = PAINLEVELRANGELIMIT.min(); lvl <= PAINLEVELRANGELIMIT.max(); lvl++) {
            String name = uName + lvl + EXT
            if (!(name in fileNames)) {
                if (!(uName in missingNames)) missingNames.add(name)
            }
        }
    }
    if (!missingNames.isEmpty()) {
        show("ERROR! Missing sound files: " + missingNames)
        showButton("Quit")
        exit()
    }
    Collections.sort(uniqueNames)
    return uniqueNames
}

def getProgressBar = { def value, def colRange = [[255, 255, 255], [255, 255, 255]], def colBG = [80, 80, 80], int charLen = PAINLEVELRANGELIMIT.max(), int charWidth = 1 ->
    String bt = ""
    String blockChar = "&#x2587;" * charWidth
    List colRangeRed = [colRange[0][0], colRange[1][0]]
    List colRangeGreen = [colRange[0][1], colRange[1][1]]
    List colRangeBlue = [colRange[0][2], colRange[1][2]]
    List colGradientRed = []
    List colGradientGreen = []
    List colGradientBlue = []

    float deltaR = (colRangeRed[1] - colRangeRed[0]) / (charLen - 1)
    float deltaG = (colRangeGreen[1] - colRangeGreen[0]) / (charLen - 1)
    float deltaB = (colRangeBlue[1] - colRangeBlue[0]) / (charLen - 1)
    for (int i = 0; i < charLen; i++) {
        colGradientRed.add((int) (colRangeRed[0] + i * deltaR))
        colGradientGreen.add((int) (colRangeGreen[0] + i * deltaG))
        colGradientBlue.add((int) (colRangeBlue[0] + i * deltaB))
    }

    float val = (value * charLen) / 100
    for (int i = 0; i < charLen; i++) {
        if (i < val) bt += "<span style='color: rgb(" +
                colGradientRed[i] + "," +
                colGradientGreen[i] + "," +
                colGradientBlue[i] + ");'>" + blockChar + "</span>"
        else bt += "<span style='color: rgb(" + colBG.toString().replaceAll("\\[|\\]", "") + ");'>" + blockChar + "</span>"
    }
    return bt
}

def getRandomEstimFileName = { return estimFileNames[getRandom(estimFileNames.size())] }

def shock = { stime, plvl, boolean canStop = false, float stopAppearTime = 0.0 ->
    String sound = SOUNDSDIR + getRandomEstimFileName() + plvl + EXT
    try {
        playBackgroundSound(sound)
        if (canStop) {
            wait(stopAppearTime)
            if ((stime - stopAppearTime) != showButton("STOP SHOCK", (stime - stopAppearTime))) {
                playBackgroundSound(null)
                return false // false = missed
            }
        } else wait(stime)
        playBackgroundSound(null)
        return true
    }
    catch (Exception e) {
        playBackgroundSound(null)
        openCdTrays()
        show("Error! Sound file missing?")
        showButton("Quit")
        exit()
    }
}

def testEstim = {
    String sound = ""
    int estimIndex = 0
    List<String> testEstimFileNames = getEstimFileNames()
    float stime = 1.0
    int testPainLevel = PAINLEVELRANGELIMIT[1]
    List eggQ = []
    while (true) {
        String txt = "Make sure no other audio sources will play!\n" +
                "Setup your your eStim device properly!\n" +
                "\n<b>WARNING!\nPressing 'TEST' will play an eStim audio file immediately for 1 second!</b>" +
                "\n\n<span style='font-size: 12px';')>audio files:\n"
        for (int i = 0; i < testEstimFileNames.size(); i++) {
            if (i == estimIndex) txt += "<b>" + testEstimFileNames[i] + ".wav</b>\n"
            else txt += testEstimFileNames[i] + ".wav\n"
        }
        txt += "</span>"

        int menu = getSelectedValue(txt, ["End Test",
                                          "Cycle",
                                          "Pain Level +1",
                                          "Pain Level -1",
                                          "TEST Pain Level " + testPainLevel,
        ])
        if (menu == 0) break
        if (menu == 1) estimIndex = (estimIndex + 1) % testEstimFileNames.size()
        if (menu == 2) if (testPainLevel < PAINLEVELRANGELIMIT[1]) testPainLevel++
        if (menu == 3) if (testPainLevel > PAINLEVELRANGELIMIT[0]) testPainLevel--
        if (menu == 4) {
            sound = SOUNDSDIR + testEstimFileNames[estimIndex] + testPainLevel + EXT
            playBackgroundSound(sound)
            if (stime != showButton("STOP!", stime)) playBackgroundSound(null)
            playBackgroundSound(null)
            eggQ.add(testPainLevel)
            if (eggQ == [6, 9, 6, 9, 6, 9]) finalPunishmentMultiRange = [0.69, 0.69]
        }
    }
}

def getParameterText = {
    String txt
    txt = "<p style='font-family:monospace;font-size:12px;'>"
    if (applyShocksRange[0] == applyShocksRange[1]) txt += "Shocks to apply ................. " + applyShocksRange[0] + "\n"
    else txt += "Shocks to apply ................. " + applyShocksRange[0] + " to " + applyShocksRange[1] + " (random)\n"
    txt += "Apply Time ...................... " + applyTimeRange[0] + " to " + applyTimeRange[1] + " seconds\n"
    txt += "Delay Time ...................... " + delayTimeRange[0] + " to " + delayTimeRange[1] + " seconds\n"
    if (painLevelRange[0] == painLevelRange[1]) txt += "Pain level ...................... " + painLevelRange[0] + "\n"
    else txt += "Pain level ...................... " + painLevelRange[0] + " to " + painLevelRange[1] + "\n"
    txt += "Shock time ...................... " + shockTimeRange[0] + " to " + shockTimeRange[1] + " seconds\n" +
            "Auto apply shocks ............... " + ((autoApply) ? "auto" : "manual") + "\n" +

            "\nPenalties:\n" +
            "Shocks to apply ................. +" + applyShocksPenalty + "\n" +
            "Pain Level ...................... "
    if (penaltyLimit && (painLevelRange[0] == painLevelRange[1])) txt += "+0 (limits active)\n"
    else txt += "+" + penaltyPainLevelPlus + " (max. " + PAINLEVELRANGELIMIT.max() + ")\n"
    txt += "Shock Time ...................... "
    if (penaltyLimit && (shockTimeRange[0] == shockTimeRange[1])) txt += "+0 (limits active)\n"
    else txt += "+" + penaltyShockTimePlus + " (max. " + SHOCKTIMERANGELIMIT.max() + ")\n"
    txt += "Penalty Shocks .................. " + penaltyShocks + "\n" +
            "Penalty Shocks Increase ......... +" + penaltyShocksPlus + "\n"
    if (pauseTimeRange[0] == pauseTimeRange[1]) txt += "Pause between shocks ............ " + pauseTimeRange[0] + "\n"
    else txt += "Pause between shocks ............ " + pauseTimeRange[0] + " to " + pauseTimeRange[1] + " seconds\n"
    txt += "\nFinal punishment ................ "
    if (finalPunishment == FINALPUNISHMENTS[0]) txt += "after all missed applied shocks"
    else if (finalPunishment == FINALPUNISHMENTS[1]) txt += "<b>Yes</b>"
    else txt += "None"
//    txt += "\nFinal punishment multiplier ..... x" + finalPunishmentMulti
    if (finalPunishmentMultiRange[0] == finalPunishmentMultiRange[1]) txt += "\nFinal punishment multiplier ..... x" + finalPunishmentMultiRange[0] + "\n"
    else txt += "\nFinal punishment multiplier ..... x" + finalPunishmentMultiRange[0] + " to " + finalPunishmentMultiRange[1] + " (random)\n"
    if (finalPunishment != FINALPUNISHMENTS[0]) txt += "\n<b>Warning! Program ends only after all applied shocks!</b>"
    txt += "</p>"
    return txt
}

def resetDefault = {
    applyShocksRange = APPLYSHOCKSRANGE
    applyTimeRange = APPLYTIMERANGE
    painLevelRange = PAINLEVELRANGE
    shockTimeRange = SHOCKTIMERANGE
    pauseTimeRange = PAUSETIMERANGE
    delayTimeRange = DELAYTIMERANGE
    finalPunishmentMultiRange = FINALPUNISHMENTMULTIRANGE

    applyShocksPenalty = APPLYSHOCKPENALTY
    penaltyShocks = PENALTYSHOCKS
    penaltyShocksPlus = PENALTYSHOCKSPLUS
    penaltyPainLevelPlus = PENALTYPAINLEVELPLUS
    penaltyShockTimePlus = PENALTYSHOCKTIMEPLUS
    penaltyLimit = PENALTYLIMIT
    finalPunishment = FINALPUNISHMENTS[0]
    estimFileNames = getEstimFileNames()
    autoApply = AUTOAPPLY
}

def loadConfig = {
    List<String> estimNames = getEstimFileNames()

    applyShocksRange.clear()
    applyTimeRange.clear()
    painLevelRange.clear()
    shockTimeRange.clear()
    pauseTimeRange.clear()
    delayTimeRange.clear()
    for (int i = 0; i <= 1; i++) {
        applyShocksRange.add(loadInteger("ShockYourself.applyShocksRange." + i))
        applyTimeRange.add(loadFloat("ShockYourself.applyTimeRange." + i))
        painLevelRange.add(loadInteger("ShockYourself.painLevelRange." + i))
        shockTimeRange.add(loadFloat("ShockYourself.shockTimeRange." + i))
        pauseTimeRange.add(loadFloat("ShockYourself.pauseTimeRange." + i))
        delayTimeRange.add(loadFloat("ShockYourself.delayTimeRange." + i))
        finalPunishmentMultiRange.add(loadFloat("ShockYourself.finalPunishmentMultiRange." + i))
    }
    applyShocksPenalty = loadInteger("ShockYourself.applyShocksPenalty")
    penaltyShocks = loadInteger("ShockYourself.penaltyShocks")
    penaltyShocksPlus = loadInteger("ShockYourself.penaltyShocksPlus")
    penaltyPainLevelPlus = loadInteger("ShockYourself.penaltyPainLevelPlus")
    penaltyShockTimePlus = loadFloat("ShockYourself.penaltyShockTimePlus")
    penaltyLimit = loadBoolean("ShockYourself.penaltyLimit")
    finalPunishment = loadString("ShockYourself.finalPunishment")
    for (int i = 0; i < estimNames.size(); i++) {
        String estimName = loadString("ShockYourself.estimFileNames." + i)
        if (estimName != null) estimFileNames.add(estimName)
    }
    autoApply = loadBoolean("ShockYourself.autoApply")

    if (applyShocksRange.any { e -> e == null }) applyShocksRange = APPLYSHOCKSRANGE
    if (applyTimeRange.any { e -> e == null }) applyTimeRange = APPLYTIMERANGE
    if (painLevelRange.any { e -> e == null }) painLevelRange = PAINLEVELRANGE
    if (shockTimeRange.any { e -> e == null }) shockTimeRange = SHOCKTIMERANGE
    if (pauseTimeRange.any { e -> e == null }) pauseTimeRange = PAUSETIMERANGE
    if (delayTimeRange.any { e -> e == null }) delayTimeRange = DELAYTIMERANGE
    if (finalPunishmentMultiRange.any { e -> e == null }) finalPunishmentMultiRange = FINALPUNISHMENTMULTIRANGE

    if (applyShocksPenalty == null) applyShocksPenalty = APPLYSHOCKPENALTY
    if (penaltyShocks == null) penaltyShocks = PENALTYSHOCKS
    if (penaltyShocksPlus == null) penaltyShocksPlus = PENALTYSHOCKSPLUS
    if (penaltyPainLevelPlus == null) penaltyPainLevelPlus = PENALTYPAINLEVELPLUS
    if (penaltyShockTimePlus == null) penaltyShockTimePlus = PENALTYSHOCKTIMEPLUS
    if (penaltyLimit == null) penaltyLimit = PENALTYLIMIT
    if (finalPunishment == null) finalPunishment = FINALPUNISHMENTS[0]
    if (estimFileNames.isEmpty()) estimFileNames = estimNames
    if (autoApply == null) autoApply = AUTOAPPLY
}

def saveConfig = {
    save("ShockYourself.applyShocksRange", applyShocksRange)
    save("ShockYourself.applyTimeRange", applyTimeRange)
    save("ShockYourself.painLevelRange", painLevelRange)
    save("ShockYourself.shockTimeRange", shockTimeRange)
    save("ShockYourself.pauseTimeRange", pauseTimeRange)
    save("ShockYourself.delayTimeRange", delayTimeRange)
    save("ShockYourself.applyShocksPenalty", applyShocksPenalty)
    save("ShockYourself.penaltyShocks", penaltyShocks)
    save("ShockYourself.penaltyShocksPlus", penaltyShocksPlus)
    save("ShockYourself.penaltyPainLevelPlus", penaltyPainLevelPlus)
    save("ShockYourself.penaltyShockTimePlus", penaltyShockTimePlus)
    save("ShockYourself.penaltyLimit", penaltyLimit)
    save("ShockYourself.finalPunishment", finalPunishment)
    save("ShockYourself.finalPunishmentMultiRange", finalPunishmentMultiRange)
    save("ShockYourself.estimFileNames", estimFileNames)
    save("ShockYourself.autoApply", autoApply)

    // old values
    save("ShockYourself.finalPunishmentMulti", null)
}

def changeConfig = {
    String txt

    def changeApplyShocksRange = {
        while (true) {
            String rangeText = applyShocksRange[0] + "-" + applyShocksRange[1]
            txt = "How many shocks do you want to apply to yourself?" +
                    "\n\ncurrent range: " + rangeText
            String strRange = getString(txt, rangeText)
            def min
            def max
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    min = range[0]
                    max = range[1]
                    if (min.isInteger() && max.isInteger()) {
                        min = min.toInteger()
                        max = max.toInteger()
                    }
                }
            } else {
                if (strRange.isInteger()) {
                    min = strRange.toInteger()
                    max = min
                }
            }
            if (min > 0 && max > 0 && min <= max) {
                applyShocksRange[0] = min
                applyShocksRange[1] = max
                break
            }
        }
    }

    def changeApplyTimeRange = {
        while (true) {
            String rangeText = applyTimeRange[0] + "-" + applyTimeRange[1]
            txt = "Set the range how long you have to apply a shock." +
                    "\n\ncurrent range: " + rangeText
            String strRange = getString(txt, rangeText)
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    def min = range[0]
                    def max = range[1]
                    if (min.isFloat() && max.isFloat()) {
                        min = min.toFloat()
                        max = max.toFloat()
                        if (min > 0 && max > 0) {
                            applyTimeRange[0] = min
                            applyTimeRange[1] = max
                            break
                        }
                    }
                }
            }
        }
    }

    def changePainLevelRange = {
        while (true) {
            String rangeText = painLevelRange[0] + "-" + painLevelRange[1]
            txt = "Set the pain level range from 1 to 10." +
                    "\n\ncurrent range: " + rangeText +
                    "\n\n<b>Warning! Pain difference can be huge!</b>"
            String strRange = getString(txt, rangeText)
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    def min = range[0]
                    def max = range[1]
                    if (min.isInteger() && max.isInteger()) {
                        min = min.toInteger()
                        max = max.toInteger()
                        if (min >= PAINLEVELRANGELIMIT[0] && min <= PAINLEVELRANGELIMIT[1] &&
                                max >= PAINLEVELRANGELIMIT[0] && max <= PAINLEVELRANGELIMIT[1]) {
                            painLevelRange[0] = min
                            painLevelRange[1] = max
                            break
                        }
                    }
                }
            }
        }
    }

    def changeShockTimeRange = {
        while (true) {
            String rangeText = shockTimeRange[0] + "-" + shockTimeRange[1]
            txt = "Set the range how long a single shock will run." +
                    "\n\ncurrent range: " + rangeText +
                    "\n\nmaximum is " + SHOCKTIMERANGELIMIT[1]
            String strRange = getString(txt, rangeText)
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    def min = range[0]
                    def max = range[1]
                    if (min.isFloat() && max.isFloat()) {
                        min = min.toFloat()
                        max = max.toFloat()
                        if (min >= SHOCKTIMERANGELIMIT[0] && min <= SHOCKTIMERANGELIMIT[1] &&
                                max >= SHOCKTIMERANGELIMIT[0] && max <= SHOCKTIMERANGELIMIT[1]) {
                            shockTimeRange[0] = min
                            shockTimeRange[1] = max
                            break
                        }
                    }
                }
            }
        }
    }

    def changeAutoApply = {
        txt = "Apply shocks will be apply automatically? or have to be applied be hand." +
                "\n\ncurrent setting: " + ((autoApply) ? "auto" : "manual") +
                "\n\n(default: manual)"
        autoApply = getBoolean(txt, "auto", "manual")
    }

    def changeApplyShocksPenalty = {
        while (true) {
            txt = "Number of shocks added to number of shocks per failed applied shock as a penalty."
            applyShocksPenalty = getInteger(txt, applyShocksPenalty)
            if (applyShocksPenalty >= 0) break
        }
    }

    def changePenaltyShocks = {
        while (true) {
            txt = "Penalty shocks per failed applied shock."
            penaltyShocks = getInteger(txt, penaltyShocks)
            if (penaltyShocks >= 0) break
        }
    }

    def changePauseTimeRange = {
        while (true) {
            String rangeText = pauseTimeRange[0] + "-" + pauseTimeRange[1]
            txt = "Set the range how long a pause between shocks will be." +
                    "\n\ncurrent range: " + rangeText
            String strRange = getString(txt, rangeText)
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    def min = range[0]
                    def max = range[1]
                    if (min.isFloat() && max.isFloat()) {
                        min = min.toFloat()
                        max = max.toFloat()
                        if (min >= 0 && max >= 0) {
                            pauseTimeRange[0] = min
                            pauseTimeRange[1] = max
                            break
                        }
                    }
                }
            }
        }
    }

    def changeDelayTimeRange = {
        while (true) {
            String rangeText = delayTimeRange[0] + "-" + delayTimeRange[1]
            txt = "Delay time before shocks can be applied." +
                    "\n\ncurrent range: " + rangeText
            String strRange = getString(txt, rangeText)
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    def min = range[0]
                    def max = range[1]
                    if (min.isFloat() && max.isFloat()) {
                        min = min.toFloat()
                        max = max.toFloat()
                        if (min >= 0 && max >= 0) {
                            delayTimeRange[0] = min
                            delayTimeRange[1] = max
                            break
                        }
                    }
                }
            }
        }
    }

    def changePenaltyShocksPlus = {
        while (true) {
            txt = "Penalty shocks increase."
            penaltyShocksPlus = getInteger(txt, penaltyShocksPlus)
            if (penaltyShocksPlus >= 0) break
        }
    }

    def changePenaltyPainLevelPlus = {
        while (true) {
            txt = "Penalty pain level increase."
            penaltyPainLevelPlus = getInteger(txt, penaltyPainLevelPlus)
            if (penaltyPainLevelPlus >= 0) break
        }
    }

    def changePenaltyShockTimePlus = {
        while (true) {
            txt = "Penalty shock time increase."
            penaltyShockTimePlus = getFloat(txt, penaltyShockTimePlus)
            if (penaltyShockTimePlus >= 0) break
        }
    }

    def changePenaltyLimit = {
        txt = "Set penalty limits active?\n\nA penalty can incrase the pain level and shock time over your set limits.\n" +
                "\nPain Level: " + painLevelRange.min() + " to " + painLevelRange.max() +
                "\nShock Time: " + shockTimeRange.min() + " to " + shockTimeRange.max() + " seconds" +
                "\n\ncurrent setting: " + ((penaltyLimit) ? "limits active" : "no limits")
        penaltyLimit = getBoolean(txt)
    }

    def changeFinalPunishment = {
        txt = "Final punishment: " +
                "after " + applyShocksRange[0] + " to " + applyShocksRange[1] + " missed self applied shocks, " +
                "after all applied shocks, or none."
        finalPunishment = FINALPUNISHMENTS[getSelectedValue(txt, FINALPUNISHMENTS)]
    }

    def changeFinalPunishmentMultiRange = {
        while (true) {
            String rangeText = finalPunishmentMultiRange[0] + "-" + finalPunishmentMultiRange[1]
            txt = "Final punishment multiplier (random)" +
                    "\n\ncurrent range: " + rangeText +
                    "\n\n<b>Warning! High values can make the final punishment run for very long time with hundreds of shocks!"
            String strRange = getString(txt, rangeText)
            def min
            def max
            if (strRange.indexOf('-') >= 0) {
                List range = strRange.split('-').collect()
                if (range.size() == 2) {
                    min = range[0]
                    max = range[1]
                    if (min.isFloat() && max.isFloat()) {
                        min = min.toFloat()
                        max = max.toFloat()
                    }
                }
            } else {
                if (strRange.isFloat()) {
                    min = strRange.toFloat()
                    max = min
                }
            }
            if (min >= FINALPUNISHMENTMULTIRANGE.min() && max >= FINALPUNISHMENTMULTIRANGE.max() && min <= max) {
                finalPunishmentMultiRange[0] = min
                finalPunishmentMultiRange[1] = max
                break
            }
        }
    }

    def changeEstimFileNames = {
        while (true) {
            List estimNames = getEstimFileNames()
            List checkBox = [false] * estimNames.size()
            int i = 0
            estimNames.each { e ->
                if (e in estimFileNames) checkBox[i] = true
                i++
            }
            checkBox = getBooleans("eStim sound files:", estimNames, checkBox)
            if (!checkBox) break
            estimFileNames.clear()
            i = 0
            estimNames.each { e ->
                if (checkBox[i]) estimFileNames.add(e)
                i++
            }
            if (!estimFileNames.isEmpty()) break
        }
    }

    boolean bConf = true
    while (bConf) {
        int menu = getSelectedValue(getParameterText(), [
                "Shocks to apply",
                "Apply Time",
                "Delay Time",
                "Pain level",
                "Shock time",
                "Auto Apply Shocks: " + ((autoApply) ? "auto" : "manual"), // case 5
                "Penalty: Shocks to apply +",
                "Penalty Shocks",
                "Penalty Shocks Increase",
                "Pause between shocks",
                "Penalty Pain Level Increase", // case 10
                "Penalty Shock Time Increase",
                "Penalty Limit: " + ((penaltyLimit) ? "limits active" : "no limits"),
                "Final punishment",
                "Final punishment multiplier",
                "eStim sound files",
                "----------------",
                "Reset to default", // case 17
                "Save & Exit",
                "Exit",
        ])
        switch (menu) {
            case 0:
                changeApplyShocksRange() //changeApplyShocks()
                break
            case 1:
                changeApplyTimeRange()
                break
            case 2:
                changeDelayTimeRange()
                break
            case 3:
                changePainLevelRange()
                break
            case 4:
                changeShockTimeRange()
                break
            case 5:
                changeAutoApply()
                break
            case 6:
                changeApplyShocksPenalty()
                break
            case 7:
                changePenaltyShocks()
                break
            case 8:
                changePenaltyShocksPlus()
                break
            case 9:
                changePauseTimeRange()
                break
            case 10:
                changePenaltyPainLevelPlus()
                break
            case 11:
                changePenaltyShockTimePlus()
                break
            case 12:
                changePenaltyLimit()
                break
            case 13:
                changeFinalPunishment()
                break
            case 14:
                changeFinalPunishmentMultiRange()
                break
            case 15:
                changeEstimFileNames()
                break
        // no case 16 ----------------
            case 17:
                resetDefault()
                break
            case 18: // Save & Exit
                saveConfig()
                bConf = false
                break
            case 19: // Exit
                bConf = false
                break
        }
    }
}

def playGame = {
    // game vars
    int applyShocks = getRandom(applyShocksRange.max() - applyShocksRange.min() + 1) + applyShocksRange.min()
    int applyShocksTotal = applyShocks
    int appliedShocks = 0
    int missedShocks = 0
    int totalPenaltyShocks = penaltyShocks
    int countPenaltyShocks = 0
    int countFinalShocks = 0
    int painLevel = painLevelRange[0]
    float shockTime = shockTimeRange[0]
    float applyTime = applyTimeRange[0]
    float pauseTime = pauseTimeRange[0]
    float delayTime = delayTimeRange[0]
    boolean finalPunishmentActive = false
    int applyTimeIndex = 0
    int painLevelIndex = 0
    int shockTimeIndex = 0
    int pauseTimeIndex = 0
    int delayTimeIndex = 0
    List shockTimeList = []
    List applyTimeList = []
    List painLevelList = []
    List pauseTimeList = []
    List delayTimeList = []
    int fPunMin = (int) finalPunishmentMultiRange.min() * 100
    int fPunMax = (int) finalPunishmentMultiRange.max() * 100
    float finalPunishmentMulti = (float) (getRandom(fPunMax - fPunMin) + fPunMin) / 100
    int finalShocks = (int) Math.round(finalPunishmentMulti * ((missedShocks) +(applyShocksTotal - appliedShocks)))

    float deltaApplyTime
    float deltaPainLevel
    float deltaShockTime
    float deltaPauseTime
    float deltaDelayTime

    def showStats = {
        def getDots = { String w, int len -> return "." * (len - w.size()) }

        List statsTextList = [
                ["Shocks Applied", [appliedShocks, applyShocksTotal], applyShocksTotal],
                (finalPunishment == FINALPUNISHMENTS[0]) ?
                        ["Missed", [missedShocks, applyShocks], applyShocks, [[220, 220, 0], [220, 0, 0]]] :
                        ["Missed", missedShocks],
                ["\nNext Shock:"],
                ["· Apply Time", applyTime],
                ["· Delay Time", delayTime],
                ["· Pain Level", [painLevel, PAINLEVELRANGELIMIT[1]], null, [[220, 220, 0], [220, 0, 0]]],
                ["· Shock Time", [shockTime, SHOCKTIMERANGELIMIT[1]], null, [[220, 220, 0], [220, 0, 0]]],
                ["\nPenalties:"],
                ["· Shocks to apply", applyShocksPenalty, null, " +"],
                ["· Pain Level", penaltyPainLevelPlus, null, " +"],
                ["· Shock Time", penaltyShockTimePlus, null, " +"],
                ["· Penalty Shocks", [countPenaltyShocks, totalPenaltyShocks], totalPenaltyShocks],
                ["· Penalty Shocks", penaltyShocksPlus, null, " +"],
                ["\nFinal Punishment:"],
                (finalPunishment != FINALPUNISHMENTS[2]) ?
                        ["· Shocks", [countFinalShocks, finalShocks], finalShocks, null, (" (x" + finalPunishmentMulti + ")")] :
                        ["· Shocks", "None"]
        ]
        int statsTextLen = 0
        statsTextList.each { e ->
            int s = e[0].size()
            if (s > statsTextLen) statsTextLen = s
        }
        statsTextLen += 3

        String st
        st = "<p style='font-family:monospace;font-size:12px;'>"
        statsTextList.each { e ->
            if (e.size() == 1) {
                st += e[0]
            } else {
                st += e[0] + " " + getDots(e[0], statsTextLen) + " "
                if (e[1].class == java.util.ArrayList) {
                    if (e[1][1] == 0) {
                        st += getProgressBar(0)
                    } else {
                        if (e[3]) st += getProgressBar((e[1][0] / e[1][1]) * 100, e[3])
                        else st += getProgressBar((e[1][0] / e[1][1]) * 100)
                    }
                    st += " " + e[1][0]
                    if (e[2] != null) st += "/" + e[2]
                } else {
                    if (e[3] != null) st += e[3]
                    st += e[1]
                }
                for (int i = 4; i < e.size(); i++) if (e[i]) st += e[i]
            }
            st += "\n"
        }
        st += "</p>"
        show(st)
    }

    if (applyShocks == 1) {
        applyTimeList = [applyTimeRange[1]] * applyShocks
        painLevelRange = [painLevelRange[1]] * applyShocks
        shockTimeRange = [shockTimeRange[1]] * applyShocks
        pauseTimeRange = [pauseTimeRange[1]] * applyShocks
        delayTimeRange = [delayTimeRange[1]] * applyShocks
    } else {
        deltaApplyTime = (applyTimeRange[1] - applyTimeRange[0]) / (applyShocks - 1)
        deltaPainLevel = (painLevelRange[1] - painLevelRange[0]) / (applyShocks - 1)
        deltaShockTime = (shockTimeRange[1] - shockTimeRange[0]) / (applyShocks - 1)
        deltaPauseTime = (pauseTimeRange[1] - pauseTimeRange[0]) / (applyShocks - 1)
        deltaDelayTime = (delayTimeRange[1] - delayTimeRange[0]) / (applyShocks - 1)
        for (int i = 0; i < applyShocks; i++) {
            applyTimeList.add(Math.round(((i * deltaApplyTime) + applyTimeRange[0]) * 10) / 10)
            painLevelList.add((int) Math.round(((i * deltaPainLevel) + painLevelRange[0])))
            shockTimeList.add(Math.round(((i * deltaShockTime) + shockTimeRange[0]) * 10) / 10)
            pauseTimeList.add(Math.round(((i * deltaPauseTime) + pauseTimeRange[0]) * 10) / 10)
            delayTimeList.add(Math.round(((i * deltaDelayTime) + delayTimeRange[0]) * 10) / 10)
        }
    }

    while (!finalPunishmentActive) {
        applyTime = applyTimeList[applyTimeIndex]
        painLevel = painLevelList[painLevelIndex]
        shockTime = shockTimeList[shockTimeIndex]
        pauseTime = pauseTimeList[pauseTimeIndex]
        delayTime = delayTimeList[delayTimeIndex]

        showStats()

        boolean applied = false
        wait(delayTime)
        if (autoApply) {
            wait(applyTime)
            applied = shock(shockTime, painLevel, true)
        } else {
            if (applyTime != showButton("APPLY SHOCK", applyTime)) {
                applied = shock(shockTime, painLevel, true, 0.5)
            }
        }
        if (applied) appliedShocks++

        if (!applied) { // missed
            if (totalPenaltyShocks > 0) {
                missedShocks++
                applyShocksTotal += applyShocksPenalty
                for (int i = 0; i < totalPenaltyShocks; i++) {
                    countPenaltyShocks++
                    showStats()
                    shock(shockTime, painLevel)
                    if (i < (totalPenaltyShocks - 1)) wait(pauseTime)
                }
                totalPenaltyShocks += penaltyShocksPlus
                countPenaltyShocks = 0
            }
            if (penaltyPainLevelPlus > 0) {
                for (int i = 0; i < painLevelList.size(); i++) {
                    int pl = painLevelList[i] + penaltyPainLevelPlus
                    if (penaltyLimit) {
                        if (pl <= painLevelRange.max()) painLevelList[i] = pl
                        else painLevelList[i] = painLevelRange.max()
                    } else {
                        if (pl <= PAINLEVELRANGELIMIT.max()) painLevelList[i] = pl
                        else painLevelList[i] = PAINLEVELRANGELIMIT.max()
                    }
                }
            }
            if (penaltyShockTimePlus > 0) {
                for (int i = 0; i < shockTimeList.size(); i++) {
                    float st = shockTimeList[i] + penaltyShockTimePlus
                    if (penaltyLimit) {
                        if (st <= shockTimeRange.max()) shockTimeList[i] = st
                        else shockTimeList[i] = shockTimeRange.max()
                    } else {
                        if (st <= SHOCKTIMERANGELIMIT.max()) shockTimeList[i] = st
                        else shockTimeList[i] = SHOCKTIMERANGELIMIT.max()
                    }
                }
            }
        }
        if (applyTimeIndex < applyTimeList.size() - 1) applyTimeIndex++
        if (painLevelIndex < painLevelList.size() - 1) painLevelIndex++
        if (shockTimeIndex < shockTimeList.size() - 1) shockTimeIndex++
        if (pauseTimeIndex < pauseTimeList.size() - 1) pauseTimeIndex++
        if (delayTimeIndex < delayTimeList.size() - 1) delayTimeIndex++
        finalShocks = (int) Math.round(finalPunishmentMulti * ((missedShocks) +(applyShocksTotal - appliedShocks)))

        showStats()

        if (appliedShocks >= applyShocksTotal) {
            finalPunishmentActive = true
        } else {
            if (finalPunishment == FINALPUNISHMENTS[0] && missedShocks >= applyShocks)
                finalPunishmentActive = true
        }
    }

    // final punishment
    if (finalPunishmentActive && finalPunishment != FINALPUNISHMENTS[2] && finalShocks > 0) {
        if (finalShocks == 1) {
            painLevelList = [painLevelRange[1]] * finalShocks
            shockTimeList = [shockTimeRange[1]] * finalShocks
            pauseTimeList = [pauseTimeRange[1]] * finalShocks
        } else {
            painLevelList.clear()
            shockTimeList.clear()
            pauseTimeList.clear()
            deltaPainLevel = (painLevel - painLevelRange[0]) / (finalShocks - 1)
            deltaShockTime = (shockTime - shockTimeRange[0]) / (finalShocks - 1)
            deltaPauseTime = (pauseTimeRange[1] - pauseTimeRange[0]) / (finalShocks - 1)
            for (int i = 0; i < finalShocks; i++) {
                painLevelList.add((int) Math.round(((i * deltaPainLevel) + painLevelRange[0])))
                shockTimeList.add(Math.round(((i * deltaShockTime) + shockTimeRange[0]) * 10) / 10)
                pauseTimeList.add(Math.round(((i * deltaPauseTime) + pauseTimeRange[0]) * 10) / 10)
            }
        }

        wait(1)
        for (int i = 0; i < finalShocks; i++) {
            painLevel = painLevelList[i]
            shockTime = shockTimeList[i]
            pauseTime = pauseTimeList[i]
            countFinalShocks++
            showStats()
            shock(shockTime, painLevel)
            if (i < (finalShocks - 1)) wait(pauseTime)
        }
    }
    showStats()
    wait(1)
    showButton("End Punishment Program")
}

def showIntro = {
    String txt = "<b>Shock Yourself</b> is a punishment program where you have to apply shocks to yourself.\n" +
            "If you fail to apply a shock, you will be punished!\n" +
            "If you stop a shock, you will be punished!\n\n" +
            "Test eStim first!\n\n" +
            "Attach the electrodes and Start Punishment Program!\n\n" +
            getParameterText()
    int menu = getSelectedValue(show(txt), ["Start", "Start with CD tray and Countdown", "Change Variables", "Test eStim"])
    if (menu == 0) playGame()
    if (menu == 1) {
        int countDown = 0
        if (getBoolean("Countdown before start?")) countDown = getInteger("Set countdown in minutes!", countDown)

        txt = "Open CD tray to place key?" +
                "\n\nKeep one hand free! You need to press buttons!" +
                "\n\n<b>Do not trust CD trays! Always have backups!</b>"
        if (finalPunishment != FINALPUNISHMENTS[0]) txt += "\n\n<b>Warning! Program ends only after all applied shocks!</b>"
        boolean openCDTray = getBoolean(txt)
        if (openCDTray) {
            openCdTrays()
            wait(2)
            showButton("Key placed and CD tray closed")
        }

        if (countDown != 0) {
            for (int c = countDown; c >= 1; c--) {
                String txtMinutes = " minutes."
                if (c <= 1) txtMinutes = " minute."
                show("Punishment Program starts in less than " + c + txtMinutes)
                if (60 != showButton("Start Punishment Program", 60)) break
            }
        } else {
            for (int c = 5; c >= 1; c--) {
                show("Punishment Program starts in " + c + " seconds.")
                wait(1)
            }
        }
        playGame()
        if (openCDTray) openCdTrays()
    } else if (menu == 2) changeConfig()
    else if (menu == 3) testEstim()
}


show(null)
setImage(null)
loadConfig()
while (true) {
    playBackgroundSound(null)
    showIntro()
}


/*
 * Resources
 * scripts/ShockYourself.groovy
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level10.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level11.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level12.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level13.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level14.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level15.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level16.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level17.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level18.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level19.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level1.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level20.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level2.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level3.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level4.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level5.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level6.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level7.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level8.wav
 * sounds/ShockYourself/burst_sawtooth_100hz_10sec_level9.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level10.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level11.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level12.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level13.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level14.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level15.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level16.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level17.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level18.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level19.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level1.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level20.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level2.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level3.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level4.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level5.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level6.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level7.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level8.wav
 * sounds/ShockYourself/burst_sine_100hz_10sec_level9.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level10.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level11.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level12.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level13.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level14.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level15.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level16.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level17.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level18.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level19.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level1.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level20.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level2.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level3.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level4.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level5.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level6.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level7.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level8.wav
 * sounds/ShockYourself/burst_square_100hz_10sec_level9.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level10.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level11.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level12.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level13.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level14.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level15.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level16.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level17.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level18.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level19.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level1.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level20.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level2.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level3.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level4.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level5.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level6.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level7.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level8.wav
 * sounds/ShockYourself/sawtooth_100hz_10sec_level9.wav
 * sounds/ShockYourself/sine_100hz_10sec_level10.wav
 * sounds/ShockYourself/sine_100hz_10sec_level11.wav
 * sounds/ShockYourself/sine_100hz_10sec_level12.wav
 * sounds/ShockYourself/sine_100hz_10sec_level13.wav
 * sounds/ShockYourself/sine_100hz_10sec_level14.wav
 * sounds/ShockYourself/sine_100hz_10sec_level15.wav
 * sounds/ShockYourself/sine_100hz_10sec_level16.wav
 * sounds/ShockYourself/sine_100hz_10sec_level17.wav
 * sounds/ShockYourself/sine_100hz_10sec_level18.wav
 * sounds/ShockYourself/sine_100hz_10sec_level19.wav
 * sounds/ShockYourself/sine_100hz_10sec_level1.wav
 * sounds/ShockYourself/sine_100hz_10sec_level20.wav
 * sounds/ShockYourself/sine_100hz_10sec_level2.wav
 * sounds/ShockYourself/sine_100hz_10sec_level3.wav
 * sounds/ShockYourself/sine_100hz_10sec_level4.wav
 * sounds/ShockYourself/sine_100hz_10sec_level5.wav
 * sounds/ShockYourself/sine_100hz_10sec_level6.wav
 * sounds/ShockYourself/sine_100hz_10sec_level7.wav
 * sounds/ShockYourself/sine_100hz_10sec_level8.wav
 * sounds/ShockYourself/sine_100hz_10sec_level9.wav
 * sounds/ShockYourself/square_100hz_10sec_level10.wav
 * sounds/ShockYourself/square_100hz_10sec_level11.wav
 * sounds/ShockYourself/square_100hz_10sec_level12.wav
 * sounds/ShockYourself/square_100hz_10sec_level13.wav
 * sounds/ShockYourself/square_100hz_10sec_level14.wav
 * sounds/ShockYourself/square_100hz_10sec_level15.wav
 * sounds/ShockYourself/square_100hz_10sec_level16.wav
 * sounds/ShockYourself/square_100hz_10sec_level17.wav
 * sounds/ShockYourself/square_100hz_10sec_level18.wav
 * sounds/ShockYourself/square_100hz_10sec_level19.wav
 * sounds/ShockYourself/square_100hz_10sec_level1.wav
 * sounds/ShockYourself/square_100hz_10sec_level20.wav
 * sounds/ShockYourself/square_100hz_10sec_level2.wav
 * sounds/ShockYourself/square_100hz_10sec_level3.wav
 * sounds/ShockYourself/square_100hz_10sec_level4.wav
 * sounds/ShockYourself/square_100hz_10sec_level5.wav
 * sounds/ShockYourself/square_100hz_10sec_level6.wav
 * sounds/ShockYourself/square_100hz_10sec_level7.wav
 * sounds/ShockYourself/square_100hz_10sec_level8.wav
 * sounds/ShockYourself/square_100hz_10sec_level9.wav
*/
